using System;
using System.Collections.Generic;
using System.Text;

namespace org.wayoda.csiow {

    internal class IowDevice {
        internal const int PID_IOW24=0x1501;
        internal const int PID_IOW24PV_1=0x1511;
        internal const int PID_IOW24PV_2=0x1512;
        internal const int PID_IOW40=0x1500;
        internal const int PID_IOW56=0x1503;
        private const String NAME_IOW24="IOWarrior24";
        private const String NAME_IOW24PV="IOWarrior24PV";
        private const String NAME_IOW40="IOWarrior40";
        private const String NAME_IOW56="IOWarrior56";

        private IowHandle handle;
        private String productName;
        private int reportSizeIO;
        private int reportSizeSM;
        private int deviceLatency;
        private IowSpecialMode[] ism;

        internal IowDevice(IowHandle handle) {
            this.handle=handle;
            if(handle.getProduct()==PID_IOW24) {
                reportSizeIO=3;
                reportSizeSM=8;
                deviceLatency=8;
                productName=NAME_IOW24;
                if(handle.getRevision()>=0x1021) {
                    IowSpecialMode[] sm={IowSpecialMode.IO, IowSpecialMode.IIC, IowSpecialMode.LCD, IowSpecialMode.SPI,
				      IowSpecialMode.RC5, IowSpecialMode.LED_MATRIX, IowSpecialMode.PIN_STATUS};
                    ism=sm;
                }
                else {
                    IowSpecialMode[] sm={IowSpecialMode.IO, IowSpecialMode.IIC, IowSpecialMode.LCD, IowSpecialMode.SPI,
				      IowSpecialMode.RC5, IowSpecialMode.PIN_STATUS};
                    ism=sm;
                }
            }
            else if(handle.getProduct()==PID_IOW40) {
                reportSizeIO=5;
                reportSizeSM=8;
                deviceLatency=8;
                productName=NAME_IOW40;
                if(handle.getRevision()>=0x1021) {
                    IowSpecialMode[] sm={IowSpecialMode.IO, IowSpecialMode.IIC, IowSpecialMode.LCD, 
				      IowSpecialMode.LED_MATRIX, IowSpecialMode.SWITCH_MATRIX, IowSpecialMode.PIN_STATUS};
                    ism=sm;
                }
                else {
                    IowSpecialMode[] sm={ IowSpecialMode.IO, IowSpecialMode.IIC, IowSpecialMode.LCD, IowSpecialMode.PIN_STATUS };
                    ism=sm;
                }
            }
            else if(handle.getProduct()==PID_IOW56) {
                reportSizeIO=8;
                reportSizeSM=64;
                deviceLatency=1;
                productName=NAME_IOW56;
                IowSpecialMode[] sm={IowSpecialMode.IO, IowSpecialMode.IIC, IowSpecialMode.LCD, IowSpecialMode.SPI,
				  IowSpecialMode.SWITCH_MATRIX, IowSpecialMode.LED_MATRIX, IowSpecialMode.PIN_STATUS};
                ism=sm;
            }
        }

        internal bool fileNameMatch(String devFileName) {
            return handle.fileNameMatch(devFileName);
        }

        internal bool isUnplugged() {
            return handle.isUnplugged();
        }

        internal void setUnplugged() {
            handle.setUnplugged();
        }

        internal int getFileHandleIO() {
            return handle.getFileHandleIO();
        }

        internal int getFileHandleSM() {
            return handle.getFileHandleSM();
        }

        internal int getProductId() {
            return handle.getProduct();
        }

        internal String getProductIdString() {
            return String.Format("{0:X4}", handle.getProduct());
        }

        internal String getProductName() {
            return productName;
        }

        internal uint getSerial() {
            return handle.getSerial();
        }

        internal String getSerialString() {
            return String.Format("{0:X8}", handle.getSerial());
        }

        internal int getRevision() {
            return handle.getRevision();
        }

        internal String getRevisionString() {
            int i1, i2, i3, i4, rev;

            StringBuilder sb=new StringBuilder();
            rev=handle.getRevision();
            i4=rev%16;
            rev=rev/16;
            i3=rev%16;
            rev=rev/16;
            i2=rev%16;
            rev=rev/16;
            i1=rev%16;
            sb.Append(String.Format("{0:X}", i1));
            sb.Append(".");
            sb.Append(String.Format("{0:X}", i2));
            sb.Append(".");
            sb.Append(String.Format("{0:X}", i3));
            sb.Append(".");
            sb.Append(String.Format("{0:X}", i4));
            return sb.ToString();
        }

        internal IowSpecialMode[] getSupportedSpecialModes() {
            IowSpecialMode[] modes=new IowSpecialMode[ism.Length];
            for(int i=0;i<ism.Length;i++)
                modes[i]=ism[i];
            return modes;
        }

        internal bool isSupportedMode(IowSpecialMode mode) {
            for(int i=0;i<ism.Length;i++) {
                if(ism[i]==mode)
                    return true;
            }
            return false;
        }

        internal int getReportSizeIO() {
            return reportSizeIO;
        }

        internal int getReportSizeSM() {
            return reportSizeSM;
        }

        internal int getDeviceLatency() {
            return deviceLatency;
        }

        public override String ToString() {
            StringBuilder sb=new StringBuilder();
            sb.Append(productName);
            sb.Append(" [product=");
            sb.Append(getProductIdString());
            sb.Append(",serial=");
            sb.Append(getSerialString());
            sb.Append(",revision=");
            sb.Append(getRevisionString());
            sb.Append("]");
            return sb.ToString();
        }
    }
}


