using System;
using System.Collections;
using System.Text;

namespace org.wayoda.csiow {
    /// <summary>
    /// A filter for IOWarriors that the library should use.
    /// </summary>
    public sealed class IowDeviceFilter {
        // The list of devices we accept
        private ArrayList al=new ArrayList();

        //an entry in the list of devices
        private struct DeviceId {
            internal int product;
            internal uint serial;

            public DeviceId(int product, uint serial) {
                this.product=product;
                this.serial=serial;
            }
        }
        /// <summary>
        /// Creates a new IowDeviceFilter that accepts every IOWarrior.
        /// </summary>
        public IowDeviceFilter() {
        }

        /// <summary>
        /// Add a specific IOWarrior to be accepted by the filter.
        /// </summary>
        /// <param name="product">The product-id of the IOWarrior</param>
        /// <param name="serial">The serial-number of the IOWarrior</param>
        public void add(int product, uint serial) {
            if(product!=IOWarrior.PID_IOW24 
               && product!=IOWarrior.PID_IOW24PV_1
                && product!=IOWarrior.PID_IOW24PV_2
                && product!=IOWarrior.PID_IOW40
                && product!=IOWarrior.PID_IOW56)
                throw new ArgumentException("Product-Id is invalid ["
                    +String.Format("{0:x4}", product)+"]");
            DeviceId id=new DeviceId(product, serial);
            al.Add(id);
        }

        /// <summary>
        /// Add a product to be accepted by the filter. The library will
        /// open every IOWarrior with a matching product-id.
        /// </summary>
        /// <param name="product">The product-id of the IOWarrior</param>
        public void addProduct(int product) {
            add(product, 0);
        }

        /// <summary>
        /// Test wether the library will accept this IOWarrior
        /// </summary>
        /// <param name="product">The product-id of the IOWarrior
        /// </param>
        /// <param name="serial">The serial-number of the IOWarrior</param>
        /// <returns>
        /// True if the library will open this device, false otherwise.
        /// </returns>
        public bool accept(int product, uint serial) {
            DeviceId id;
            if(al.Count==0)
                return true;
            for(int i=0;i<al.Count;i++) {
                id=(DeviceId)al[i];
                if(id.product==product) {
                    if(id.serial==0 || id.serial==serial)
                        return true;
                }
            }
            return false;
        }

    }
}
