using System;
using System.Collections.Generic;
using System.Text;

namespace org.wayoda.csiow {
    /// <summary>
    /// The arguments for an Event triggered by new data received from
    /// an IOWarrior.
    /// </summary>
    public class IowReportEventArgs : EventArgs {
        private IowSpecialMode mode;
        private IowReport report;

        /// <summary>
        /// Creates a new IowReportEventArgs object.
        /// </summary>
        /// <param name="mode">
        /// One of the IowSpecialMode constants that specify 
        /// the type of report
        /// </param>
        /// <param name="report">
        /// The data received from the IOWarrior
        /// </param>
        public IowReportEventArgs(IowSpecialMode mode, IowReport report) {
            this.mode=mode;
            this.report=report;
        }

        /// <summary>
        /// Gets the SpecialMode for the data
        /// </summary>
        /// <returns>
        /// One of the typed constants for SpecialModes 
        /// </returns>
        public IowSpecialMode getSpecialMode() {
            return mode;
        }

        /// <summary>
        /// The actual data received from the IOWarrior
        /// </summary>
        /// <returns>
        /// The report from the device
        /// </returns>
        public IowReport getReport() {
            return report;
        }

        public override String ToString() {
            StringBuilder sb=new StringBuilder();
            sb.Append("SpecialMode : "+mode.ToString());
            sb.Append("\r\n");
            sb.Append(report.ToString());
            return sb.ToString();
        }
      }
}
