using System;
using System.Collections.Generic;
using System.Text;

namespace org.wayoda.csiow {
    /// <summary>
    /// This interface needs to be implemented by an application that
    /// wants to be notified when new data was reported by an IOWarrior.
    /// </summary>
    public interface IowReportListener {
        /// <summary>
        /// This method will be called when a new report was read
        /// from an IOWarrior.
        /// </summary>
        /// <param name="report">
        /// The report that was read from the IOWarrior.
        /// </param>
        /// <param name="iow">
        /// The IOWarrior that is the source of the report.
        /// </param>
        void reportUpdate(IowReport report, IOWarrior iow);

        /// <summary>
        /// This method will be called when an IOWarrior to which
        /// we are listening is unplugged, so no more reports are to be expected
        /// from this device.
        /// </summary>
        /// <param name="iow">
        /// The IOWarrior that was unplugged
        /// </param>
        void deviceRemoved(IOWarrior iow);
    }
}
