using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using org.wayoda.csiow;

namespace org.wayoda.csiow {
    /// <summary>
    /// An internal invisible window that is used to track
    /// WM_DEVICECHANGE messages.
    /// </summary>
    internal sealed class MessageCatcher : NativeWindow {
        DeviceChangeEventHandler handler;
        internal static readonly int DEVICE_REMOVED=0;
        internal static readonly int DEVICE_ADD=1;

        /// <summary>
        /// Creates a new invisible window. 
        /// </summary>
        /// <param name="handler">
        /// The delegate to be notified when device-changes
        /// are detected.
        /// </param>
        internal MessageCatcher(DeviceChangeEventHandler handler) {
            CreateParams cp=new CreateParams();
            cp.Caption = "Hidden window";
            cp.ClassName = null;
            cp.X = 0x7FFFFFFF;
            cp.Y = 0x7FFFFFFF;
            cp.Height = 0;
            cp.Width = 0;
            cp.Style = 0x02000000; //WS_CLIPCHILDREN;
            try {
                CreateHandle(cp);
            }
            catch(System.ComponentModel.Win32Exception e) {
                //we don't care why we failed but...
                throw new IowManager.IowManagerException(new IowError(IowError.Type.LIB_ERROR, null,
                    "Unable to create internal window for devicechange-detection", e.NativeErrorCode));
            }
            this.handler=handler;
        }

        /// <summary>
        /// Listens for WM_DEVICECHANGE messages and reports them to
        /// the listener through a delegate.
        /// </summary>
        /// <param name="m">The windows-message</param>
        protected override void WndProc(ref Message m) {
            int wparam=m.WParam.ToInt32();
            int lparam=m.LParam.ToInt32();
            if(m.Msg==HidDevice.WM_DEVICECHANGE) {
                if(wparam==HidDevice.DBT_DEVICEARRIVAL) {
                    handler(MessageCatcher.DEVICE_ADD);
                }
                else if(wparam==HidDevice.DBT_DEVICEREMOVECOMPLETE) {
                    handler(MessageCatcher.DEVICE_REMOVED);
                }
            }
            else
                base.WndProc(ref m);
        }

    }
}